import * as pdfJSLib from 'pdfjs-dist'
import pdfJsWorker from 'pdfjs-dist/build/pdf.worker.entry'
import {getCurrentInstance, markRaw, nextTick, onUnmounted, reactive, ref, Ref, toRaw, watch, watchEffect} from 'vue'
import {fabric} from 'fabric'
import {IEvent} from 'fabric/fabric-impl'
import {DataRectangle, DrawRectangle, LabeledRect, SelectionIEvent} from '@/types'
import FabricLabeledRect from '@/fabric/labeled.rect'
import rectangleDefaults from '@/fabric/rectangleDefaults'
import useUpdateInternalModel from '@/composables/useUpdateInternalModel'
import Color from 'color'
import {useDebounceFn} from "@vueuse/core";
import useStore from "@/store/useStore";

export default function useInitFabric(htmlCanvas: Ref<HTMLCanvasElement | undefined>, modelValue: Ref<DataRectangle[][] | undefined>) {
    const instance = getCurrentInstance()
    const emit = instance?.emit
    const scale = window.devicePixelRatio
    const totalPages = ref(1)

    const getPdfCanvas = async (file: string, pageNum: number, zoom: number) => {
        pdfJSLib.GlobalWorkerOptions.workerSrc = pdfJsWorker
        const loadingTask = pdfJSLib.getDocument(file)

        const pdf = await loadingTask.promise
        const page = await pdf.getPage(pageNum)
        const canvas = document.createElement('canvas')
        const context = canvas.getContext('2d') as CanvasRenderingContext2D
        const viewport = page.getViewport({scale: zoom})
        canvas.height = viewport.height
        canvas.width = viewport.width
        const renderContext = {
            canvasContext: context,
            viewport: viewport,
        }

        viewport.width = canvas.width
        viewport.height = canvas.height

        totalPages.value = pdf.numPages

        if (emit) {
            emit('total-pages', pdf.numPages)
            emit('current-page', pageNum)
        }
        // For some reason, using await page.render(renderContext) causes a blank page so using promise instead
        return page.render(renderContext).promise.then(() => canvas)
    }

    let canvas = ref<fabric.Canvas | undefined>()
    let currentFile = ''
    let currentPage = 1
    let currentZoom = 1
    let currentWidth = 0
    let currentHeight = 0

    const setFile = async (canvas: fabric.Canvas, file: string, pageNum: number, zoom: number) => {
        let pdfCanvas

        if (currentFile !== file || currentPage !== pageNum) {
            unmountWatchEffect()
            resetModelPage(pageNum)

            currentPage = pageNum
            const currentChildren = canvas.getObjects()
            canvas.remove(...currentChildren)
            currentFile = file
            pdfCanvas = await getPdfCanvas(file, pageNum, zoom * scale)
            currentWidth = pdfCanvas.width / zoom / scale
            currentHeight = pdfCanvas.height / zoom / scale

            canvas.setDimensions({width: currentWidth, height: currentHeight})
            const pdfImage = new fabric.Image(pdfCanvas)
                .scaleToWidth(currentWidth)
                .set('selectable', false)
                .set('evented', false)

            canvas.add(pdfImage)
        }
    }

    const setZoom = (canvas: fabric.Canvas, zoom: number, doForce = false) => {
        if (currentZoom !== zoom || doForce) {
            const newZoom = zoom
            const zoomBy = newZoom === currentZoom ? newZoom : newZoom / currentZoom

            const width = canvas.width ? canvas.width * zoomBy : 0
            const height = canvas.height ? canvas.height * zoomBy : 0
            currentWidth = width
            currentHeight = height

            canvas.setDimensions({width, height})
            canvas.setZoom(newZoom)
            currentZoom = zoom
        }
    }

    const resetZoom = (canvas: fabric.Canvas) => {
        const width = canvas.width || 0
        const height = canvas.height || 0
        currentWidth = width
        currentHeight = height
        canvas.setDimensions({width, height})
        canvas.setZoom(1)
        currentZoom = 1
    }

    const handleSelection = (event: SelectionIEvent | null) => {
        if (emit) {
            if (event?.selected.length) {
                emit('object-selected', event.selected[0])
            } else {
                emit('object-selected', null)
            }
        }
    }
    const {resetModelPage} = useStore()

    const {setInternalModel} = useUpdateInternalModel(canvas)

    const setInternalModelDebounced = useDebounceFn(() => {
        setInternalModel(currentPage);
    }, 200)
    const initListeners = (canvas: fabric.Canvas) => {
        canvas.on('mouse:dblclick', (event: IEvent) => {
            if (emit && event?.target) {
                emit('object-dblclick', event?.target)
            }
        })
        canvas.on('selection:created', (event: IEvent) => handleSelection(event as SelectionIEvent))
        canvas.on('selection:updated', (event: IEvent) => handleSelection(event as SelectionIEvent))
        canvas.on('selection:cleared', () => handleSelection(null))
        canvas.on('object:modified', setInternalModelDebounced)
    }

    const restoreObjectsForPage = async (pageNum: number, updateModel: boolean) => {
        const rawCanvas = toRaw(canvas.value)
        const existingRectangles = rawCanvas?.getObjects('labeledRect') as LabeledRect[]
        if (!modelValue.value?.[pageNum - 1]?.length) {
            for (const rectangle of existingRectangles) {
                rawCanvas?.remove(rectangle)
            }
            setInternalModel(pageNum)
            return
        }

        for (const drawRectangle of modelValue.value[pageNum - 1]) {
            if (existingRectangles.find(rect => rect.uid === drawRectangle.uid)) {
                continue
            }
            const newData = Object.assign(rectangleDefaults, drawRectangle)
            newData.fill = Color(newData.color).alpha(.7).rgb().toString()
            delete newData.isRotatable
            const newRect = new FabricLabeledRect(reactive(newData) as DrawRectangle)
            newRect.setControlVisible('mtr', !!drawRectangle.isRotatable)
            await nextTick()
            const rawCanvas = toRaw(canvas.value)
            rawCanvas?.add(newRect)
        }
    }

    watch(modelValue, async val => {
        if (!val) {
            return
        }
        await restoreObjectsForPage(currentPage, false)

    }, {deep: true, flush: 'post'})

    const isCanvasReady = ref(false)
    // If the canvas hasn't been initialized yet, add function calls to the pool to be executed once it is
    let delayCanvasRestore = false

    let unmountWatchEffect

    const stopPool = watch(isCanvasReady, async (val) => {
        if (val) {
            await restoreObjectsForPage(currentPage, true)
            // Only need to run this once
            stopPool()
        }
    })

    onUnmounted(() => {
        toRaw(canvas.value)?.dispose()
        canvas.value = undefined
        unmountWatchEffect()
    })

    return async (file: string, pageNum: number, zoom: number) => {
        if (!htmlCanvas.value) {
            return
        }
        if (!canvas.value) {
            canvas.value = markRaw(new fabric.Canvas(htmlCanvas.value, {
                enableRetinaScaling: true,
                renderOnAddRemove: true,
                selection: false,
                uniformScaling: false,
            }))
            initListeners(toRaw(canvas.value))
        }
        unmountWatchEffect = watchEffect(() => {
            if (!modelValue.value) {
                return
            }
            if (!canvas.value || !isCanvasReady.value) {
                delayCanvasRestore = true
                return
            }
            restoreObjectsForPage(pageNum, true)
        })

        const oldPage = currentPage
        const oldFile = currentFile

        await setFile(canvas.value, file, pageNum, zoom)
        if (modelValue.value) {
            await restoreObjectsForPage(pageNum, false)
        }
        if (oldFile !== file) {
            resetZoom(canvas.value)
        } else {
            setZoom(canvas.value, zoom, oldPage !== pageNum)
        }

        isCanvasReady.value = true
        emit?.('canvas-ready')

        return canvas.value
    }
}
